require_relative 'hashing'

class HashSet
  attr_reader :count

  def initialize(num_buckets = 8)
    @store = Array.new(num_buckets) { Array.new }
    @count = 0
  end

  def insert(key)
    resize! if @count == num_buckets
    self[key] << key unless include?(key)
    @count += 1
  end

  def include?(key)
    self[key].include?(key)
  end

  def remove(key)
    self[key].delete(key)
    @count -= 1
  end

  private

  def [](num)
    @store[num.hash % num_buckets]
  end

  def num_buckets
    @store.length
  end

  def resize!
    old_store = @store
    new_num_buckets = num_buckets * 2

    @store = Array.new(new_num_buckets) { Array.new }
    @count = 0

    old_store.each do |bucket|
      bucket.each { |num| insert(num) }
    end
  end
end
